@@ -1,10 +1,11 @@
 # -*- coding: UTF-8 -*-
 app_name = 'Python Extended Debugger'
-version = '1.19'
+version = '1.20'
 URL = 'http://bashdb.sourceforge.net/pydb/'
 HELP_URL = URL + 'pydb/index.html'
 
 import sys
-assert sys.version_info >= (2, 3, 4), _("Python %s or newer required") % '2.3.4'
+assert sys.version_info>= map(int, '2.3.3'.split('.')), \
+	 _("Python %s or newer required") % '2.3.3'
 from pydb import *
 
