@@ -1,11 +1,11 @@
-"""$Id: gdb.py.in,v 1.90 2006/10/13 22:45:10 rockyb Exp $
+"""$Id: gdb.py.in,v 1.98 2006/12/10 13:26:24 rockyb Exp $
 Handles gdb-like command processing.
 (See also pydb.doc for documentation.)"""
 
 _debugger_name = 'pydb'
 
 # The name of the debugger we are currently going by.
-_version = '1.19'
+_version = '1.20'
 
 import atexit, dis, inspect, linecache, os, pprint
 import bdb, re, subcmd, sys, types
@@ -31,9 +31,20 @@ class Restart(Exception):
 
 class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
     """A debugger class for Python that resembles the gdb (GNU debugger)
-    command set."""
+    command set.
 
-    def __init__(self, completekey='tab', stdin=None, stdout=None):
+    Parameter completekey specifies the what to use for command completion.
+    pass None if no command completion is desired.
+
+    Parameters stdin and stdout specify where debugger input and output
+    are to come from.
+    
+    Parameter siglist gives a list of signals to intercept and ignore.
+    None means the default set. If you want *no* signals, use [].
+    """
+
+    def __init__(self, completekey='tab', stdin=None, stdout=None,
+                 siglist=None):
         pydbbdb.Bdb.__init__(self)
         pydbcmd.Cmd.__init__(self, completekey, stdin, stdout)
 
@@ -42,7 +53,8 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self._wait_for_mainpyfile = False
 
         # set up signal handling
-        self.sigmgr          = sighandler.SignalManager(self)
+        self.sigmgr          = sighandler.SignalManager(self,
+                                                        ignore_list=siglist)
         self._reset_handler  = None
 
         self.__init_info()   # Initialize 'info' subcommands
@@ -132,6 +144,14 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
             self.hist_save = False
             self.set_history_length = readline.set_history_length
             self.set_history_length(256)
+
+            # An application like ipython may have its own
+            # history set up beforehand, clear that so we don't back up
+            # into that. Note that ipython saves and restores its own
+            # history.
+            if hasattr(readline, "clear_history"):
+                readline.clear_history()
+
             # Read history file and set up to write history file when we
             # exit.
             global _debugger_name
@@ -146,6 +166,8 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
 
         except ImportError:
             self.histfile = None
+            return
+        return
 
     def break_anywhere_gdb(self, frame):
         """Decorate bdb break_anyhwere to consider the sigcheck and
@@ -154,7 +176,7 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         return self.break_anywhere(frame)
 
     ### FIXME: trace_dispatch could be a performance bottleneck. If
-    ### so, consider reassigning the reoutine when don't have any
+    ### so, consider reassigning the routine when don't have any
     ### signals registered.
             
     def trace_dispatch_gdb(self, frame, event, arg):
@@ -252,6 +274,7 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         completions = self.all_completions(arg)
         if completions is None: return
         for complete in completions:  self.msg(complete)
+        return
 
     def __init_info(self):
         """Initialize info subcommands. Note: instance variable name
@@ -290,6 +313,8 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.setcmds.add('prompt',          self.set_prompt)
         self.setcmds.add('sigcheck',        self.set_sigcheck)
         ## self.setcmds.add('target-address',  self.set_target_address)
+        self.setcmds.add('warnoptions',     self.set_warnoptions)
+        return
 
     def __init_show(self):
         """Initialize show subcommands. Note: instance variable name
@@ -313,6 +338,7 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.showcmds.add('sigcheck',       self.show_sigcheck)
         ## self.showcmds.add('target-address', self.show_target_address)
         self.showcmds.add('version',        self.show_version)
+        self.showcmds.add('warnoptions',    self.show_warnoptions)
 
     # To be overridden in derived debuggers
     def defaultFile(self):
@@ -333,6 +359,7 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
             except ValueError:
                 self.msg('index %r is not a number' % i)
                 continue
+        return
 
     def lineinfo(self, identifier):
         failed = (None, None, None)
@@ -372,6 +399,7 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.stack = []
         self.curindex = 0
         self.curframe = None
+        return
 
     def interaction(self, frame, traceback):
         """Possibly goes into loop to read debugger commands."""
@@ -390,6 +418,7 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         if not do_loop:
             # Tell cmdloop to break out of its loop.
             return True
+        return False
 
     def parse_list_cmd(self, arg):
         """Parses arguments for the "list" command and returns the tuple
@@ -450,6 +479,7 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
         self.stopframe = self.botframe
         self.returnframe = None
         self.quitting = 0
+        return
 
     def setup(self, frame, traceback=None):
         """Initialization done before entering the debugger-command
@@ -466,7 +496,8 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
             self.curframe = self.stack[self.curindex][0]
         else:
             self.stack = self.curframe = self.botframe = None
-        if self.execRcLines()==1: return 1
+        if self.execRcLines()==1: return True
+        return False
 
     def setup_source(self, arg, showError=False):
         """Set up to read commands from a source file"""
@@ -481,6 +512,7 @@ class Gdb(Bdb, Cmd, SubcmdInfo, SubcmdSet, SubcmdShow):
             for line in rcFile.readlines():
                 self.rcLines.append(line)
             rcFile.close()
+        return
 
     def show_commands(self, arg):
         """Show the history of commands you typed.
@@ -540,12 +572,14 @@ the previous command number shown. A negative number starts from the end."""
         "Show debugger's prompt"
         self.msg("""%s's prompt is "%s".""" %
                  (_debugger_name, self.prompt))
+        return
 
     def show_version(self, args):
         """Show what version of the debugger this is"""
         global _version
         self.msg("""%s version %s.""" %
                  (_debugger_name, _version))
+        return
 
     def write_history_file(self):
         """Write the command history file -- possibly."""
@@ -673,7 +707,9 @@ the .py suffix may be omitted."""
         if not arg:
             self.errmsg("Argument required (new working directory).")
         else:
-           os.chdir(arg)
+            os.chdir(arg)
+            return
+        return
 
     def do_clear(self, arg):
         """cl(ear) {[file:]linenumber | function}
@@ -693,7 +729,7 @@ the .py suffix may be omitted."""
 
         if not self.curframe:
             self.msg("No frame selected.")
-            return
+            return False
         if not arg:
             frame, lineno = self.stack[self.curindex]
             filename = self.canonic_filename(self.curframe)
@@ -708,11 +744,11 @@ the .py suffix may be omitted."""
                     lineno = int(arg)
                 except:
                     self.errmsg("Invalid line number (%s)" % arg)
-                    return
+                    return False
             else:
                 (funcname, filename, lineno) = get_brkpt_lineno(self, arg)
 
-            if lineno is None: return
+            if lineno is None: return False
 
             # FIXME This default setting doesn't match that used in
             # do_break. Perhaps one is non-optimial.
@@ -727,7 +763,9 @@ the .py suffix may be omitted."""
             else:
                 self.msg("Deleted breakpoints " +
                          ' '.join(map(lambda b: str(b), brkpts)))
-        return
+                return False
+            return False
+        return False
 
     do_cl = do_clear
 
@@ -744,10 +782,10 @@ the .py suffix may be omitted."""
                 bnum = self.get_pos_int(arg, min=0, default=None,
                                         cmdname="'commands'")
             except ValueError:
-                return
+                return False
         if not (bnum < len(bdb.Breakpoint.bpbynumber)):
             self.errmsg('No breakpoint numbered %d.' % bnum)
-            return
+            return False
 
         self.commands_bnum = bnum
         self.commands[bnum] = []
@@ -757,7 +795,7 @@ the .py suffix may be omitted."""
         self.commands_defining = True
         if self.setup(self.curframe) == 1:
             self.commands_defining = False
-            return
+            return False
 
         prompt_back = self.prompt
         self.prompt = '>'
@@ -768,6 +806,7 @@ the .py suffix may be omitted."""
         self.cmdloop()
         self.commands_defining = False
         self.prompt = prompt_back
+        return False
 
     def do_condition(self, arg):
         """condition bpnumber str_condition
@@ -793,7 +832,7 @@ the .py suffix may be omitted."""
             bp = bdb.Breakpoint.bpbynumber[bpnum]
         except IndexError:
             self.errmsg("No breakpoint number %d." % bpnum)
-            return
+            return False
         if bp:
             bp.cond = cond
             if not cond:
@@ -874,6 +913,7 @@ position before running an (unconditional) continue."""
         self.print_location()
         sys.settrace(self.trace_dispatch)
         self.lastcmd = p.lastcmd
+        return False
 
     def do_delete(self, arg):
         """delete [bpnumber [bpnumber...]]  - Delete some breakpoints.
@@ -985,6 +1025,7 @@ the default."""
             sys.stdout = orig_stdout
             raise # reraise exception
         sys.stdout = orig_stdout
+        return False
 
     def do_display(self, arg):
         """display [format] EXP
@@ -1011,7 +1052,7 @@ the default."""
             dp = DisplayNode(self.curframe, variable, format)
             res = dp.checkValid(self.curframe)
             self.msg(res)
-        return
+        return False
 
     def do_down(self, arg):
         """d(own) [count]
@@ -1030,6 +1071,7 @@ the default."""
         if self.gdb_dialect:
             count = -count
         self.__adjust_frame(pos=-count, absolute_pos=False)
+        return False
 
     def do_enable(self, arg):
         """enable [display] bpnumber [bpnumber ...]]
@@ -1423,19 +1465,19 @@ assignment to the variable __return__."""
             self.errmsg('Not yet returned!')
     do_rv = do_retval
 
-    def do_run(self, arg):
-        """run [args...]
+    def do_run(self, arg_str):
+        """run [arg_str]
 
-        Run/Restart the debugged Python program. If a string is
+        Run/Restart the debugged Python program. If a string (arg_str) is
         supplied that becomes the new command arguments.  History,
         breakpoints, actions and debugger options are preserved."""
         if not self._program_sys_argv:
             self.msg("No Python program registered.")
-            # FIXME add FILE command and mention that.
+            self.msg("Perhaps you want to use the 'file' command?")
             return
-        if arg:
+        if arg_str:
             argv_start = self._program_sys_argv[0:1]
-            self._program_sys_argv = arg.split(" ")
+            self._program_sys_argv = arg_split(arg_str)
             self._program_sys_argv[:0] = argv_start
 
         raise Restart
@@ -1445,8 +1487,11 @@ assignment to the variable __return__."""
     def do_set(self, arg):
         """See help_set"""
         args = arg.split()
+        if len(args) == 1 and 'warnoptions'.startswith(args[0]):
+            self.setcmds.do(self, args[0], args)
+            return
         if len(args) < 2:
-            self.errmsg("Expecting at least 2 arguments. Got %d" % len(args))
+            self.errmsg("Expecting at least 2 arguments, got %d." % len(args))
             return
         self.setcmds.do(self, args[0], args)
 
@@ -1610,7 +1655,7 @@ Prints the type of the argument which can be a Python expression."""
                 self.errmsg("Name Error: %s" % arg)
             else:
                 self.errmsg("%s: %s" % (exc_type_name, self._saferepr(v)))
-            return
+            return False
         if inspect.ismethod(value):
             self.msg('method %s%s' %
                      (value.func_code.co_name,
@@ -1618,7 +1663,7 @@ Prints the type of the argument which can be a Python expression."""
             if inspect.getdoc(value):
                 self.msg('%s:\n%s' %
                          (value, inspect.getdoc(value)))
-            return
+            return False
         elif inspect.isfunction(value):
             self.msg('function %s%s' %
                      (value.func_code.co_name,
@@ -1626,9 +1671,10 @@ Prints the type of the argument which can be a Python expression."""
             if inspect.getdoc(value):
                 self.msg('%s:\n%s' %
                          (value, inspect.getdoc(value)))
-            return
+            return False
         # None of the above...
         self.msg(type(value))
+        return False
 
     def do_where(self, arg):
         """where [count]
@@ -1642,12 +1688,13 @@ Prints the type of the argument which can be a Python expression."""
         try:
             count = self.get_pos_int(arg, default=None, cmdname="where")
         except ValueError:
-            return
+            return False
 
         if not self.curframe:
             self.msg("No stack.")
-            return
+            return False
         print_stack_trace(self, count)
+        return False
 
     do_T = do_bt = do_where
 
@@ -1729,6 +1776,7 @@ was reached.
         command with a 'global' command, e.g.:
         %sglobal list_options; list_options = ['-l']
         %s""" % (self.prompt, self.prompt))
+        return
 
     # Note: format of help is compatible with ddd.
     def help_info(self, args):
@@ -1737,6 +1785,8 @@ was reached.
             self.infocmds.help(self, '')
         else:
             self.infocmds.help(self, args[0])
+            return
+        return
 
     def help_pydb(self, *arg):
         help()
@@ -1749,6 +1799,8 @@ You can see these environment settings with the 'show' command."""
             self.setcmds.help(self, '')
         else:
             self.setcmds.help(self, args[0])
+            return
+        return
 
     # Note: format of help is compatible with ddd.
     def help_show(self, args):
@@ -1757,10 +1809,13 @@ You can see these environment settings with the 'show' command."""
             self.showcmds.help(self, '')
         else:
             self.showcmds.help(self, args[0])
+            return
+        return
 
     def help_unalias(self):
         print """unalias name
 Deletes the specified alias."""
+        return
 
     ####### End of help section ########
 #
