@@ -1,7 +1,8 @@
 #!/usr/local/bin/python -t
-"""$Id: pydb.py.in,v 1.122 2006/10/13 22:45:10 rockyb Exp $
+"""$Id: pydb.py.in,v 1.132 2006/12/10 13:26:24 rockyb Exp $
 A Python debugger. Contains user-callable routines, e.g.
-run, set_trace, runevel.
+exception_hook, run, runl, runv, debugger (AKA set_trace), post_mortem,
+or runevel.
 
 (See also pydb.doc for documentation.)"""
 
@@ -10,7 +11,7 @@ run, set_trace, runevel.
 _debugger_name = 'pydb'
 
 # The name of the debugger we are currently going by.
-_version = '1.19'
+_version = '1.20'
 
 import inspect, os, re, sys, traceback
 
@@ -22,35 +23,23 @@ from bdb import BdbQuit
 from gdb import Gdb
 from gdb import Restart
 
-__all__ = ["Pdb",     "exception_hook", "help",
-           "pm",      "post_mortem",    "run",
-           "runcall", "runeval",        "runl",
-           "runv",    "set_trace"]
+__all__ = ["Pdb",     "debugger", "exception_hook",
+           "help",    "pm",       "post_mortem",   "run",
+           "runcall",  "runeval", "runl",          "runv",
+           "set_trace"]
 
 old_handler = None
-## _pydb_trace = None  # needs NOT to be set.
-program     = '*Unknown*'
 
-def init():
-    global program
-    global _re_linetrace_delay
-
-    if len(sys.argv) > 0:
-        program = os.path.basename(sys.argv[0])
-    else:
-        program = "*Unknown*"
-
-def process_options(p, debugger_name, prog, pkg_version,
-                    option_list=None):
+def process_options(p, debugger_name, pkg_version, option_list=None):
     """Handle debugger options. Use option_list if you want are writing
     another main program and want to extend the existing set of debugger
     options.
 
     The options dicionary from opt_parser is return. Global sys.argv is
     also updated."""
-    usage_str="""%s [debugger-options] [python-script [script-options...]]
+    usage_str="""%prog [debugger-options] [python-script [script-options...]]
 
-       Runs the extended python debugger""" % (prog)
+       Runs the extended python debugger"""
 
     optparser = OptionParser(usage=usage_str, option_list=option_list,
                              version="%%prog version %s" % pkg_version)
@@ -94,6 +83,9 @@ def process_options(p, debugger_name, prog, pkg_version,
                          + "to FILE")
 ##    optparser.add_option("--pid", dest="pid",
 ##                         help="Attach to running process PID.")
+    optparser.add_option("--sigcheck", dest="sigcheck",
+                         action="store_true", default=False, 
+                         help="Set to watch for signal handler changes")
     optparser.add_option("-t", "--target", dest="target",
                          help="Specify a target to connect to. Arguments" \
                          + " should be of form, 'protocol address'."),
@@ -162,6 +154,11 @@ def process_options(p, debugger_name, prog, pkg_version,
     if opts.execute:
         p.cmdqueue = list(opts.execute.split(';;'))
 
+    if opts.sigcheck:
+        p.cmdqueue.insert(0, "set sigcheck on")
+    else:
+        p.cmdqueue.insert(0, "set sigcheck off")
+
     if opts.output:
         try: 
             p.stdout = open(opts.output, 'w')
@@ -194,13 +191,18 @@ def process_options(p, debugger_name, prog, pkg_version,
 
 class Pdb(Gdb):
 
-    """A debugger class for Python that resembles the gdb (GNU
-    debugger) command set. Really we just inherit everything from the
-    Gdb class, but we keep the name Pdb to be compatible with the stock
-    Python debugger's (pdb's) Pdb class."""
+    """Right now this is basically the same thing as the Gdb class. In the
+    future though we may break out more stand-alone aspects and put them here
+    versues an embedded debugger which is what Gdb would be used for.
 
-    def __init__(self, completekey='tab', stdin=None, stdout=None):
-        Gdb.__init__(self, completekey, stdin, stdout)
+    See the Gdb class documentation for more information and information on
+    the meanings of the parameters.
+    """
+
+    def __init__(self, completekey='tab', stdin=None, stdout=None,
+                 siglist=None):
+        Gdb.__init__(self, completekey, stdin, stdout, siglist)
+        return
 
 # -- end class Pdb
 
@@ -208,6 +210,71 @@ class Pdb(Gdb):
 # Other top-level routines
 #########################################################
 
+def debugger(dbg_cmds=None, add_exception_hook=True, add_threaddbg=False,
+             new_instance=False):
+    """Enter the debugger at the calling stack frame.  This is useful to
+    hard-code a breakpoint at a given point in a program, even if the code
+    is not otherwise being debugged (e.g., when an assertion fails).
+    Leaving this the debugger terminates the program.
+
+    dbg_cmds is an optional list of debugger commands you want to run.
+
+    If add_exception is True (the default), we will install an exception hook
+    to call the post-mortem debugger on an unhandled exception.
+
+    If the debugger was called previously, normally we use that. This
+    way any previous debugger state that was set gets used. If you
+    don't the debugger state to be used, set parameter new_instance
+    True.
+    """
+    global _pydb_trace
+    try:
+        if not isinstance(_pydb_trace, Pdb):
+            print "Your program should not use _pydb_trace"
+            return
+    except NameError:
+        new_instance = True
+    except:
+        print "Unknown error"
+        return
+
+    if new_instance:
+        if add_threaddbg:
+            import threaddbg
+            _pydb_trace = threaddbg.threadDbg()
+        else:
+            _pydb_trace = Pdb()
+        _pydb_trace.reset()
+        _pydb_trace._program_sys_argv = list(sys.argv)
+        _pydb_trace._sys_argv = list(_pydb_trace._program_sys_argv)
+        _pydb_trace._sys_argv[:0] = [_debugger_name]
+        _pydb_trace.main_dirname = os.path.dirname(sys.argv[0])
+
+    # We don't support "run" so we'll make "run" and "R" be "restart"
+    _pydb_trace.do_R = _pydb_trace.do_run = _pydb_trace.do_restart
+    _pydb_trace.curframe=inspect.currentframe()
+
+    if _pydb_trace.running:
+        _pydb_trace.do_next(2)
+        return
+    _pydb_trace.running = True
+
+    if dbg_cmds != None:
+        _pydb_trace.cmdqueue = list(dbg_cmds)
+
+    if add_exception_hook:
+        sys.excepthook = exception_hook
+
+    # Set to skip over return statement
+    _pydb_trace.step_ignore = 1
+    # There can be no other commands after the following one
+    # unless we adjust _pydb_trace.step_ignore accordingly.
+    # Note in some cases we stop in the "return" below and in some
+    # cases we step one more so we need to be conservitive in the
+    # step count. I don't know why weird behavior.
+    _pydb_trace.set_trace(_pydb_trace.curframe)
+    return
+
 def exception_hook(type, value, tb, dbg_cmds=None, cmdfile=None):
     """An exception hook to call pydb's post-mortem debugger.
 
@@ -222,6 +289,7 @@ def exception_hook(type, value, tb, dbg_cmds=None, cmdfile=None):
     """
     traceback.print_exception(type, value, tb)
     post_mortem(tb, dbg_cmds, cmdfile, 0)
+    return
 
 # print help
 def help():
@@ -235,6 +303,7 @@ def help():
     else:
         print ("Sorry, can't find the help file '%s' along the "
                   + "Python search path") % doc_file
+    return
 
 ## def pdbserver(addr, p):
 ##     """Set up to allow a program to be debugged from outside of the
@@ -269,6 +338,7 @@ def pm(dbg_cmds=None, frameno=1):
     # in get_last_tb_or_frame_tb
     tb = fns.get_last_tb_or_frame_tb(frameno+1)
     post_mortem(tb, dbg_cmds)
+    return
 
 def post_mortem(t=None, dbg_cmds=None, cmdfile=None, frameno=1):
     """Enter debugger read loop after your program has crashed.
@@ -298,6 +368,10 @@ def post_mortem(t=None, dbg_cmds=None, cmdfile=None, frameno=1):
         # frameno+1 because we are about to add one more level of call
         # in get_last_tb_or_frame_tb
         t = fns.get_last_tb_or_frame_tb(frameno+1)
+        if t is None:
+            print "Can't find traceback for post_mortem " + \
+                  "in sys.last_traceback or sys.exec_info"
+            return
 
     # t has least-recent traceback entry first. We want the most-recent
     # entry. Also we'll pick out a mainpyfile name if it hasn't previously
@@ -350,7 +424,7 @@ def post_mortem(t=None, dbg_cmds=None, cmdfile=None, frameno=1):
                 pass
     except BdbQuit:
         pass
-      
+    return
 
 def run(statement, globals=None, locals=None):
 
@@ -369,6 +443,7 @@ def run(statement, globals=None, locals=None):
     p = Pdb()
     p.running=True
     p.run(statement, globals, locals)
+    return
 
 def runeval(expression, globals=None, locals=None):
 
@@ -413,15 +488,10 @@ def runl(*args):
 
     This function may be useful inside Python shells.
     """
-    ## FIXME: redo so we don't call main and don't have to save/restore
-    ### sys.argv              
-    sys_argv     = list(sys.argv)
-    sys.argv[0]  = 'pydb'
-    sys.argv[1:] = list(args)
-    main()
-    sys.argv     = sys_argv
+    runv(args)
+    return
 
-def runv(argv):
+def runv(argv, p=None):
 
     """Run debugger as though you typed something like
           os.system("pydb " + " ".join(argv))
@@ -440,58 +510,20 @@ def runv(argv):
     of the program to run and gets the remaining parameters.
 
     This function may be useful inside Python shells.
+
+    Parapmeter p allows you to pass in your own Pdb object.
     """
     ## FIXME: redo so we don't call main and don't have to save/restore
     ### sys.argv              
     sys_argv     = list(sys.argv)
     sys.argv[0] = 'pydb'
     sys.argv[1:] = list(argv)
-    main()
+    main(p)
     sys.argv     = sys_argv
+    return
 
-def set_trace(dbg_cmds=None, add_exception_hook=True, add_threaddbg=False):
-    """Enter the debugger at the calling stack frame.  This is useful to
-    hard-code a breakpoint at a given point in a program, even if the code
-    is not otherwise being debugged (e.g., when an assertion fails).
-    Leaving this the debugger terminates the program.
-
-    dbg_cmds is an optional list of debugger commands you want to run.
-
-    If add_exception is True (the default), we will install an exception hook
-    to call the post-mortem debugger on an unhandled exception.
-    """
-    global _pydb_trace
-    try:
-        if not isinstance(_pydb_trace, Pdb):
-            print "Your program should not use _pydb_trace"
-            return
-    except NameError:
-        if add_threaddbg:
-            import threaddbg
-            _pydb_trace = threaddbg.threadDbg()
-        else:
-            _pydb_trace = Pdb()
-        _pydb_trace.reset()
-        _pydb_trace._program_sys_argv = list(sys.argv)
-        _pydb_trace._sys_argv = list(_pydb_trace._program_sys_argv)
-        _pydb_trace._sys_argv[:0] = [_debugger_name]
-        _pydb_trace.main_dirname = os.path.dirname(sys.argv[0])
-    except:
-        print "Unknown error"
-        return
-    # We don't support "run" so we'll make "run" and "R" be "restart"
-    _pydb_trace.do_R = _pydb_trace.do_run = _pydb_trace.do_restart
-    _pydb_trace.curframe=inspect.currentframe()
-    _pydb_trace.running = True
-
-    if dbg_cmds != None:
-        _pydb_trace.cmdqueue = list(dbg_cmds)
-
-    if add_exception_hook:
-        sys.excepthook = exception_hook
-
-    # There can be no other commands after the following one:
-    _pydb_trace.set_trace(_pydb_trace.curframe)
+# This is the older name for debugger and is compatible with pdb's name.
+set_trace=debugger
 
 ## def signal_handler(signum, frame):
 ##     """ This signal handler replaces the program's signal handler
@@ -548,12 +580,12 @@ def set_trace(dbg_cmds=None, add_exception_hook=True, add_threaddbg=False):
 # Main program
 #########################################################
 
-def main():
+def main(p=None):
     """Routine which gets run if we were invoked directly"""
 
-    p = Pdb()
-    (opts, p) = process_options(p, _debugger_name, program,
-                                '1.19')
+    if p is None:
+        p = Pdb()
+    (opts, p) = process_options(p, _debugger_name, '1.20')
 
     # process_options has munged sys.argv to remove any options that
     # options that belong to this debugger. The original options to
@@ -641,9 +673,8 @@ def main():
     if p.stdout != sys.stdout:
         p.stdout.close()  # In case someone is waiting on this.
     # Restore old sys.argv
-    sys.argv = p._program_sys_argv 
-
-init()
+    sys.argv = p._program_sys_argv
+    return
 
 # When invoked as main program, invoke the debugger on a script
 if __name__=='__main__':
