@@ -1,9 +1,9 @@
-"""$Id: set.py,v 1.11 2006/09/26 02:01:10 rockyb Exp $
+"""$Id: set.py,v 1.13 2006/11/05 12:52:24 rockyb Exp $
 set subcommands, except those that need some sort of text substitution.
 (Those are in gdb.py.in.)
 """
 
-import inspect, re, sighandler
+import inspect, re, sighandler, sys
 
 class SubcmdSet:
 
@@ -213,9 +213,9 @@ signal is encountered you should set this on."""
                     self.trace_dispatch = self.trace_dispatch_gdb
                 else:
                     # Turn off signal checking/adjusting
-                    self.break_anywhere = self.break_anywhere_bdb
-                    self.set_continue   = self.set_continue_bdb
-                    self.trace_dispatch = self.trace_dispatch_bdb
+                    self.break_anywhere = self.break_anywhere_old
+                    self.set_continue   = self.set_continue_old
+                    self.trace_dispatch = self.trace_dispatch_old
             self.sigcheck = sigcheck
         except ValueError:
             pass
@@ -226,3 +226,18 @@ signal is encountered you should set this on."""
         self.target_addr = self.target_addr.strip()
         self.msg('target address set to %s' % self.target_addr)
 
+    def set_warnoptions(self, args):
+
+        """Set the Python warning options that are in effect when a
+program is started or restarted. On the command line, these are the -W
+options, e.g. -Werror, or -We::Deprecation. However options should not
+contain leading -W's and should be separated with white space only,
+e.g. don't use commas.
+
+Examples:
+  set warn error e::Deprecation
+  set warnoptions
+"""
+
+        sys.warnoptions = args[1:]
+        self.show_warnoptions(args)
