@@ -1,5 +1,5 @@
 #!/usr/bin/python
-# $Id: except.py.in,v 1.2 2006/07/11 10:53:36 rockyb Exp $
+# $Id: except.py.in,v 1.3 2006/12/07 09:29:26 rockyb Exp $
 """Test to see if handles an exception when an error is raised."""
 import os, sys
 
@@ -15,7 +15,7 @@ sys.path.insert(0, os.path.join(top_srcdir, 'pydb'))
 print __doc__
 
 import pydb
-pydb.set_trace()  # we go into the debugger
+pydb.debugger()  # we go into the debugger
 print "Returning from the debugger"
 # Set a variable so we can show the state is
 # somewhere after the above set_trace()
